# -*- coding: utf_8 -*-
# Author: Ryan Myers
# Models: Jeff Styers, Reagan Heller


# Last Updated: 6/13/2005
#
# This tutorial provides an example of creating a character
# and having it walk around on uneven terrain, as well
# as implementing a fully rotatable camera.

import direct.directbase.DirectStart
from pandac.PandaModules import CollisionTraverser,CollisionNode
from pandac.PandaModules import CollisionHandlerQueue,CollisionRay
from pandac.PandaModules import Filename
from pandac.PandaModules import PandaNode,NodePath,Camera,TextNode
from pandac.PandaModules import Vec3,Vec4,BitMask32
from direct.gui.OnscreenText import OnscreenText
from direct.actor.Actor import Actor
from direct.task.Task import Task
from direct.showbase.DirectObject import DirectObject
import random, sys, os, math



from direct.showbase.DirectObject import DirectObject
from pandac.PandaModules import *

from direct.gui.DirectGui import *

SPEED = 0.5

# Figure out what directory this program is in.
MYDIR=os.path.abspath(sys.path[0])
MYDIR=Filename.fromOsSpecific(MYDIR).getFullpath()

camCenter = render.attachNewNode('camCenter')   
base.camera.setPos(0,20,90)
base.camera.reparentTo(camCenter)
base.camera.lookAt(camCenter)


def point_dist(p1,p2):
    return math.sqrt((p1[0]-p2[0])**2+(p1[1]-p2[1])**2+(p1[2]-p2[2])**2)
class World(DirectObject):

    def __init__(self):
        
        self.keyMap = {"left":0, "right":0, "vpered":0, "cam-left":0, "cam-right":0, "nazad":0}
        base.win.setClearColor(Vec4(0,0,0,1))

        # Post the instructions

        self.environ = loader.loadModel("models/world")      
        self.environ.reparentTo(render)
        self.environ.setPos(0,0,0)
        
        # Create the main character, Ralph

      #  ralphStartPos = self.environ.find("**/start_point").getPos()
        self.ralph = Actor("models/ralph",
                                 {"run":"models/ralph-run",
                                  "walk":"models/ralph-walk"})
        self.ralph.reparentTo(render)
        self.ralph.setScale(.2)
      #  self.ralph.setPos(ralphStartPos)
        self.ralph.setPos(10,10,5)

        # Create a floater object.  We use the "floater" as a temporary


        # Accept the control keys for movement and rotation

        self.accept("escape", sys.exit)
        self.accept("a", self.setKey, ["left",1])
        self.accept("d", self.setKey, ["right",1])
        self.accept("w", self.setKey, ["vpered",1])
        self.accept("s", self.setKey, ["nazad",1])
        self.accept("arrow_left", self.setKey, ["cam-left",1])
        self.accept("arrow_right", self.setKey, ["cam-right",1])
        self.accept("a-up", self.setKey, ["left",0])
        self.accept("d-up", self.setKey, ["right",0])
        self.accept("w-up", self.setKey, ["vpered",0])
        self.accept("s-up", self.setKey, ["nazad",0])
        self.accept("arrow_left-up", self.setKey, ["cam-left",0])
        self.accept("arrow_right-up", self.setKey, ["cam-right",0])

        taskMgr.add(self.move,"moveTask")
        
        # Game state variables
        self.isMoving = False

        # Set up the camera
        
        #base.disableMouse()
        self.target=Vec3()
        self.camDist=10
        self.sens = 0.3
        self.mx,self.my=0,0
        
        #taskMgr.add(self.moveUpdate, 'move-task') ###
        self.turnCameraAroundPoint(0,0,self.target,self.camDist,self.sens)
        #base.camera.setPos(self.ralph.getX(),self.ralph.getY()+10,2)
        taskMgr.add(self.dragTask,'dragTask') 
        base.disableMouse()


        self.cTrav = CollisionTraverser()

        self.ralphGroundRay = CollisionRay()
        self.ralphGroundRay.setOrigin(0,0,1000)
        self.ralphGroundRay.setDirection(0,0,-1)
        self.ralphGroundCol = CollisionNode('ralphRay')
        self.ralphGroundCol.addSolid(self.ralphGroundRay)
        self.ralphGroundCol.setFromCollideMask(BitMask32.bit(0))
        self.ralphGroundCol.setIntoCollideMask(BitMask32.allOff())
        self.ralphGroundColNp = self.ralph.attachNewNode(self.ralphGroundCol)
        self.ralphGroundHandler = CollisionHandlerQueue()
       # self.nodeCamHandler1 = CollisionHandlerQueue()
        self.cTrav.addCollider(self.ralphGroundColNp, self.ralphGroundHandler)

        self.camGroundRay = CollisionRay()
        self.camGroundRay.setOrigin(0,-0.2,0)
        self.camGroundRay.setDirection(0,1,0)
        self.camGroundCol = CollisionNode('camRay')
        self.camGroundCol.addSolid(self.camGroundRay)
        self.camGroundCol.setFromCollideMask(BitMask32.bit(0))
        self.camGroundCol.setIntoCollideMask(BitMask32.allOff())
        self.camGroundColNp = base.camera.attachNewNode(self.camGroundCol)
        self.camGroundHandler = CollisionHandlerQueue()
        self.cTrav.addCollider(self.camGroundColNp, self.camGroundHandler)

        # Uncomment this line to see the collision rays
        #self.ralphGroundColNp.show()
        #self.camGroundColNp.show()
       
        #Uncomment this line to show a visual representation of the 
        #collisions occuring
        #self.cTrav.showCollisions(render)
       

    
    #Records the state of the arrow keys
    def setKey(self, key, value):
        self.keyMap[key] = value
    

    # Accepts arrow keys to move either the player or the menu cursor,
    # Also deals with grid checking and collision detection
    def move(self, task):
   
        startpos = self.ralph.getPos()

        # If a move-key is pressed, move ralph in the specified direction.
        if (self.keyMap["vpered"]!=0):
            self.ralph.setPos(self.ralph,0,-0.5,0)
        if (self.keyMap["nazad"]!=0):
            self.ralph.setPos(self.ralph,0,0.5,0)
        if (self.keyMap["right"]!=0):
            self.ralph.setPos(self.ralph,-0.5,0,0)
        if (self.keyMap["left"]!=0):
            self.ralph.setPos(self.ralph,0.5,0,0)

        # If ralph is moving, loop the run animation.
        # If he is standing still, stop the animation.

        if (self.keyMap["vpered"]!=0) or (self.keyMap["left"]!=0) or (self.keyMap["right"]!=0)or (self.keyMap["nazad"]!=0):
            if self.isMoving is False:
                self.ralph.loop("run")
                self.isMoving = True
        else:
            if self.isMoving:
                self.ralph.stop()
                self.ralph.pose("walk",5)
                self.isMoving = False

        # If the camera is too far from ralph, move it closer.
        # If the camera is too close to ralph, move it farther.

        

        camCenter.setPos(self.ralph.getX(),self.ralph.getY(),self.ralph.getZ()+2.1)
        # Now check for collisions.

        self.cTrav.traverse(render)

        # Adjust ralph's Z coordinate.  If ralph's ray hit terrain,
        # update his Z. If it hit anything else, or didn't hit anything, put
        # him back where he was last frame.

        entries = []
        for i in range(self.ralphGroundHandler.getNumEntries()):
            entry = self.ralphGroundHandler.getEntry(i)
            entries.append(entry)
        entries.sort(lambda x,y: cmp(y.getSurfacePoint(render).getZ(),
                                     x.getSurfacePoint(render).getZ()))
        if (len(entries)>0) and (entries[0].getIntoNode().getName() == "terrain"):
            self.ralph.setZ(entries[0].getSurfacePoint(render).getZ())
        else:
            self.ralph.setPos(startpos)

        # Keep the camera at one foot above the terrain,
        # or two feet above ralph, whichever is greater.
        near_point=Point3(0,0,0)
        #entries = []
#####            
        if self.camGroundHandler.getNumEntries()>0:
             self.camGroundHandler.sortEntries() 
             entry = self.camGroundHandler.getEntry(0) 
             near_point = entry.getSurfacePoint(render)
           #  print entry2.getSurfacePoint(render)

        p1=self.ralph.getX(),self.ralph.getY(),self.ralph.getZ()
        p2=near_point.getX(),near_point.getY(),near_point.getZ()
        
        
        
        dist=point_dist(p1,p2)
        print dist
        if (dist<3.5):
            base.camera.setY(-3.5)
            print 'EEEEEEEEEEEEEEEEEEEEEEE'
            #base.camera.setY(-dist)
        else:
            base.camera.setY(-3.5)


        return Task.cont
 #####               


    def turnCameraAroundPoint(self,tx,ty,p,dist,sn):
      #  props = WindowProperties()
      #  props.setCursorHidden(True)
      #  base.win.requestProperties(props) 
        #print 'aaa'
        Xc=base.win.getXSize()/2-base.win.getPointer(0).getX()
        Yc=base.win.getYSize()/2-base.win.getPointer(0).getY()
        base.win.movePointer(0,base.win.getXSize()/2,base.win.getYSize()/2)
        
        newCamHpr=Vec3()         
        camHpr=base.camera.getHpr()
        newCamHpr.setX(camHpr.getX()+Xc*sn) 
        newCamHpr.setY(camHpr.getY()+Yc*sn)
        newCamHpr.setZ(camHpr.getZ())
        base.camera.setHpr(newCamHpr)
        angleradiansX = newCamHpr.getX() * (math.pi / 180.0)
        angleradiansY = newCamHpr.getY() * (math.pi / 180.0)
        base.camera.setPos( dist*math.sin(angleradiansX)*math.cos(angleradiansY),
                           -dist*math.cos(angleradiansX)*math.cos(angleradiansY),
                           -dist*math.sin(angleradiansY) )
        #поворот персонажа
        self.ralph.setH(self.ralph.getH() + Xc*0.3)

    def dragTask(self,task):
            if base.mouseWatcherNode.hasMouse():
                self.dragging=True
                mpos = base.mouseWatcherNode.getMouse()  
                if self.dragging:
                    self.turnCameraAroundPoint((self.mx-mpos.getX())*100,(self.my-mpos.getY())*100,self.target,self.camDist,self.sens)        
                self.mx=mpos.getX()
                self.my=mpos.getY()     
                
            return task.cont   
w = World()
run()
